using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;

// VAMQualityPlugin v1.0
//
// Controls additional quality settings for VaM

namespace VAMTweaksPlugin
{
    public class VAMTweaks : MVRScript
    {
		public JSONStorableFloat SettingShadowDistance;
		public JSONStorableStringChooser SettingShadowCascade;
		public JSONStorableStringChooser SettingShadowProjection;

		private List<string> ShadowCascadeChoices;
		private List<int> ShadowCascadeValues;
		
		private List<string> ShadowProjectionChoices;
		private List<ShadowProjection> ShadowProjectionValues;
			
		public override void Init()
        {

			if ( containingAtom.type != "SessionPluginManager" && containingAtom.type != "Empty" )
			{
				logError("VAMTweaks can only be used has a Session plugin OR on an Empty atom.");
				return;
			}
			
			ShadowCascadeChoices = new List<string>();
			ShadowCascadeChoices.Add("0");
			ShadowCascadeChoices.Add("2");
			ShadowCascadeChoices.Add("4");
			
			ShadowCascadeValues = new List<int>();
			ShadowCascadeValues.Add(0);
			ShadowCascadeValues.Add(2);
			ShadowCascadeValues.Add(4);
			
			ShadowProjectionChoices = new List<string>();
			ShadowProjectionChoices.Add("StableFit");
			ShadowProjectionChoices.Add("CloseFit");
			
			ShadowProjectionValues = new List<ShadowProjection>();
			ShadowProjectionValues.Add(ShadowProjection.StableFit);
			ShadowProjectionValues.Add(ShadowProjection.CloseFit);
			
            try
            {
				// Creating Storables
				SettingShadowDistance = new JSONStorableFloat("Shadow Distance", 25f, SettingShadowDistanceCallback, 0f, 250f);
				
				SettingShadowCascade = new JSONStorableStringChooser("Shadow Cascade", ShadowCascadeChoices, "2", "Shadow Cascade", ShadowCascadeCallback);
				SettingShadowProjection = new JSONStorableStringChooser("Shadow Projection", ShadowProjectionChoices, "StableFit", "Shadow Projection", ShadowProjectionCallback);

				
				// Creating Storables UI
				CreateSlider( SettingShadowDistance );
				
				UIDynamicPopup SCdp = CreateScrollablePopup(SettingShadowCascade, false);
				SCdp.labelWidth = 180f;
				
				UIDynamicPopup SPdp = CreateScrollablePopup(SettingShadowProjection, false);
				SPdp.labelWidth = 180f;
				
				// Register Storables
				RegisterFloat(SettingShadowDistance);
				RegisterStringChooser(SettingShadowCascade);
				RegisterStringChooser(SettingShadowProjection);
				
				
				// Help text
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help - VAMTweaks</b></size></color>\n\n" + 
					"<color=#333>" +
					"This plugin will help you tweak some VaM settings.\n\n" +
					"<b>Shadow Distance:</b> increase the distance at which the shadow is visible. The further the shadows are visible, the less accurate they are (they might be ugly at short distances).\n" +
					"<b>Shadow Cascade:</b> changes the number of cascades used to reduce perspective aliasing. It is a setting only influencing directional lights.\n" +
					"<b>Shadow Projection:</b> changes the way shadows are projected. CloseFit renders higher resolution but might produce artifacts when the camera moves, StableFit renders a lower resolution but reduce the artifacts in motion. Only for directional lights too.\n\n" +
					"<b>For the shadow settings :</b> Keep in mind that these settings might improve OR decrease the quality depending on your unique case especially on directional lights. Putting everything at max will not produce high quality shadows, you will have to test the settings and find a good balance to get a good draw distance and a good quality.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1150.0f;
				
				
				// Initializing values
				SettingShadowDistanceCallback(SettingShadowDistance.val);
				ShadowCascadeCallback(SettingShadowCascade.val);
				ShadowProjectionCallback(SettingShadowProjection.val);
			}
            catch(Exception e)
            {
                SuperController.LogError("VAMTweaks - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			try
            {

			}
            catch(Exception e)
            {
                SuperController.LogError("VAMTweaks - Exception caught: " + e);
            }
        }
		
		void Update() {

		}
			
		// **************************
		// Callbacks & similar
		// **************************
		private void SettingShadowDistanceCallback( float dist ) {
			QualitySettings.shadowDistance = dist;
		}
		
		private void ShadowCascadeCallback( string choice ) {
			QualitySettings.shadowCascades = ShadowCascadeValues[ ShadowCascadeChoices.IndexOf(choice) ];
		}
		
		private void ShadowProjectionCallback( string choice ) {
			QualitySettings.shadowProjection = ShadowProjectionValues[ ShadowProjectionChoices.IndexOf(choice) ];
		}

			
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
		

		
		// **************************
		// Time to cleanup !
		// **************************
		void OnDestroy() {
			// Cleaning modified values

			// Restoring shadow cascades
			// I don't know why VAM's value is at 1 by default, 'coz Unity has three values : 0, 2 and 4
			// But I prefer to restore it has the original one
			QualitySettings.shadowCascades = 1;
			
			// Restoring the rest of the settings
			QualitySettings.shadowDistance = 25f;
			QualitySettings.shadowProjection = ShadowProjection.StableFit;
		}

	}
}